import { createHmac, randomBytes } from 'node:crypto'
import dayjs from 'dayjs'

import SessionStore from './sessionStore.ts'

/**
 *  Session
 * 
 *
 */
class Session {

    /**  
     *
     *
     */
    private db

    /**
     *
     *
     */
    constructor(db: object) {
        this.db = db
        this.sessionStore = new SessionStore(db)
    }

    /**
     *  create session 
     *
     *  @param number userId
     *  @param string agent
     *  @param string language
     *  @param string ip
     *  @return object
     *
     */
    public async create(userId: number, agent: string, language: string, ip: string): object {

        const userHmac = this.createUserHmac(userId)
        const users = await this.sessionStore.findOneByUser(userHmac)

        // if users are not longer can login, drop oldest
        if (users && users.length > process.env.NANO_MAX_LOGINS) {
            const session = await this.sessionStore.dropOldest(userHash)
        }

        const token = randomBytes(64).toString('hex')
        const tokenHmac = this.createTokenHmac(token, agent, language, ip)

        // create date 
        const date = dayjs()

        await this.sessionStore.create({
            'user' : userHmac,
            'token' : tokenHmac,
            'expired_at': date.add(process.env.NANO_SESSION_DURATION, 'minute').toISOString(),
            'created_at': date.toISOString()
        })

        return {
            'user': userHmac,
            'token': token // raw token goes in cookie 
        }
    }

    /**
     *  delete single session for a user
     *
     *  @param object userId
     *
     */
    public async destroy(user: object): void {
        await this.sessionStore.remove(user.id)
    }

    /**
     *  delete all session for a user
     *
     *  @param number userId
     *
     */
    public async destroyAll(userId: number): void {
        const userHash = this.createUserHmac(userId)
        const users = await this.sessionStore.findByUser(userHash)

        for (const index in users) {
            await this.destroy(users[index])
        }
    }

    /**
     *  getting single session
     *
     *  @param number userId
     *  @param string agent
     *  @param string language
     *  @param string ip
     *  @param string token
     *  @return string 
     *
     */
    public async get(userId: number, agent: string, language: string, ip: string, token: string): string {
        const userHash = this.createUserHmac(userId)
        const tokenHash = this.createTokenHmac(token, agent, language, ip)

        let user = await this.sessionStore.findOneByUserAndToken(userHash, tokenHash)

        // if user found, check expired_at, if expired, delete session
        if (user && user.expired_at) {
            if (dayjs().isAfter(dayjs(user.expired_at))) {
                await this.destroy(user)
                user = null
            }
        }

        return user
    }

    /**
     *  creating hmac for userId 
     *
     *  @param number userId
     *  @return string 
     *
     */
    private createUserHmac(userId: number): string {
        const hmac = createHmac('sha512', process.env.NANO_SESSION_SECRET)
        const buffer = new Buffer(userId)
        
        return hmac.update(buffer).digest('base64')
    }

    /**
     *  creating hmac for token 
     *
     *  @param number userId
     *  @param string agent
     *  @param string language
     *  @param string ip
     *  @return string 
     *
     */
    private createTokenHmac(token: string, agent: string, language: string, ip: string): string {
        const hmac = createHmac('sha512', process.env.NANO_SESSION_SECRET)
        const buffer = new Buffer(token + agent + language + ip)

        return hmac.update(buffer).digest('base64') 
    }
}

export default Session
